#include <iostream>
#include <cstring>
using namespace std;

class intArray {

public:
	explicit intArray(int sz, const int* p_array = nullptr) : sz{sz} {
		if(p_array) memcpy(array, p_array, sz * sizeof(int));
	}

	[[nodiscard]]
	int get_size() const { return sz; }

//	operator [] is inherited by Derived class
	int& operator[](int index) {
		return array[index];
	}

//	operator << cannot be inherited since it is not ptr member function
//	It works because of the parameter type conversion
//	(const NamedArray& -> const intArray&)
	friend ostream& operator<<(ostream& out, const intArray& p_array) {
		out << "<";
		for (int i = 0; i < p_array.sz; i++)
			out << p_array.array[i] << ((i == p_array.sz - 1) ? ">" : ", ");
		return out;
	}

protected:
	static constexpr int DIM{5};
	int sz, array[DIM]{};
};

class NamedArray : public intArray {

public:
	explicit NamedArray (int sz, const int* a = nullptr, string s = ""s) :
			intArray(sz, a), name{move(s)} {}
	void printName() {
		cout << name << ": ";
	}

private:
	string name;
};

int main () {
	int tmp_array[] {1, 3, 7, 4, 9};
	const int sz = sizeof tmp_array / sizeof tmp_array[0];

	intArray int_array(sz, tmp_array);
	cout << "intArray, [] operator\n";
	cout << "<";
	for (int i = 0; i < int_array.get_size(); i++)
		cout << int_array[i] << ((i == int_array.get_size() - 1) ? ">" : ", ");
	cout << endl;
	cout << endl;

	cout << "intArray, << operator\n";
	cout << int_array << endl;
	cout << endl;

	NamedArray named_array(sz, tmp_array, "NamedArray");
	cout << "NamedArray, [] operator\n";
	named_array.printName();
	cout << "<";
	for (int i = 0; i < named_array.get_size(); i++)
		cout << named_array[i] << ((i == named_array.get_size() - 1) ? ">" : ", ");
	cout << endl;
	cout << endl;

	cout << "NamedArray, << operator\n";
	named_array.printName();
	cout << named_array << endl;

	return 0;
}

